@@ -25,30 +25,38 @@ label.
 
 =head2 OPTIONS
 
-The following options are standard among all Glyphs.  See individual
-glyph pages for more options.
+The following options are standard among all Glyphs.  See
+L<Bio::Graphics::Glyph> for a full explanation.
 
-  Option      Description               Default
-  ------      -----------               -------
+  Option      Description                      Default
+  ------      -----------                      -------
 
-  -fgcolor    Foreground color		black
+  -fgcolor      Foreground color	       black
 
-  -outlinecolor				black
-	      Synonym for -fgcolor
+  -outlinecolor	Synonym for -fgcolor
 
-  -bgcolor    Background color          white
+  -bgcolor      Background color               turquoise
 
-  -fillcolor  Interior color of filled  turquoise
-	      images
+  -fillcolor    Synonym for -bgcolor
 
-  -linewidth  Width of lines drawn by	1
-		    glyph
+  -linewidth    Line width                     1
 
-  -height     Height of glyph		10
+  -height       Height of glyph		       10
 
-  -font       Glyph font		gdSmallFont
+  -font         Glyph font		       gdSmallFont
+
+  -connector    Connector type                 0 (false)
+
+  -connector_color
+                Connector color                black
+
+  -label        Whether to draw a label	       0 (false)
+
+  -description  Whether to draw a description  0 (false)
+
+  -strand_arrow Whether to indicate            0 (false)
+                 strandedness
 
-  -label      Whether to draw a label	false
 
 =head1 BUGS
 
